/*
SetFileInfo.c

USAGE

	SetFileInfo("Output File",'xxxx','yyyy');

ARGUMENTS

	char	*fileName;		The name of the file (including HFS
							directory name) to be set.
	
	OStype	'xxxx'			The four character file type, eg 'TEXT'

	OStype	'xxxx'			The four character creator, eg 'XCEL','CGRF','QKPT','QED1'

COMMENTS

	The syntax 'xxxx' results in four characters packed into a short int. This is
	a Macintosh convention, not Standard C. That's ok since this routine is strictly
	Macintosh anyway.

	As far as I can tell the ChangeCreatorType function in Apple's MoreFilesExtras.c
	does the same thing, but with more precautions. We probably should use it
	instead. It automatically calls FSpBumpDate, if appropriate.

EXAMPLE

	** create Cricket data file **
	myFile=fopen("LuminanceCalibration.data","w");
	SetFileInfo("LuminanceCalibration.data",'TEXT','CGRF');
	fprintf(myFile,"*\n");
	fprintf(myFile,"x\ty\n");
	for(i=0;i<n;i++)fprintf(myFile,"%9g\t%9g\n",x[i],y[i]);
	fclose(myFile);

REFERENCE:

	>From blob@ricochet.net
	Date: Sun, 01 Dec 1996 11:07:54 -0700
	Organization: (none)
	
	In article <32A1BDC9.149A@uclink4.berkeley.edu>,
	cwright@uclink4.berkeley.edu wrote:
	
	> I've written a small app to quickly change the Creator/File type of a
	> file.  It works fine, but after it changes these, the Finder does not
	> update the file until you do a Get Info or close and open the folder
	> that the file is in so that it has to reread the file info again.  
	
	This is answered in the FAQ at <http://www.best.com/~ckt/faq/>, with sample
	code in MoreFiles. (BumpDate in MoreFilesExtras.c)  You need to change the modification date of the parent
	folder, or send the scriptable Finder an event to update its display.
	
	See <http://www.best.com/~ckt/faq/Four.html#30> for details.
	
	
HISTORY
12/87	EJC wrote q_make_crick()
8/89	dgp Renamed and rewritten.
8/24/91	dgp	Made compatible with THINK C 5.0.
12/28/92 dgp Removed obsolete support for THINK C 4.
4/10/97	dgp	Eliminate stuff that was conditional on !UNIVERSAL_HEADERS.

*****************************************************************************/
#include "VideoToolbox.h"	/* for prototype of itself */
//#include <Files.h>

/*
COMMENT:
As far as I can tell the ChangeCreatorType function in Apple's MoreFilesExtras.c
does the same thing as SetFileInfo, but with more precautions. We probably should use it
instead. It automatically calls BumpDate, if appropriate.
*/
void SetFileInfo(char *fileName,OSType fileType,OSType fileCreator)
{
	FInfo outFileInfo;
	
	c2pstr(fileName);
	GetFInfo((StringPtr)fileName,0,&outFileInfo);
	outFileInfo.fdType = fileType;
	outFileInfo.fdCreator = fileCreator;
	SetFInfo((StringPtr)fileName,0,&outFileInfo);
	p2cstr((void *)fileName);
}

/*
Changing the ioDrMdDat field of your file's parent directory forces the
Finder into updating its window. This is useful after you've changed a file's
type, to get the Finder to immediately show the new icon.

HISTORY:
Craig Marciniak, TemplarDev@aol.com, published in March, 1995 issue of MacTech Magazine, p. 80.

COMMENT:
As far as I can tell the FSpBumpDate in Apple's MoreFilesExtras.c does the same thing, but with
more precautions. We probably should use it instead.
*/
#include <LowMem.h>		// LMGetTime
void ForceFinderToUpdateFileIcon(FSSpecPtr file);

void ForceFinderToUpdateFileIcon(FSSpecPtr file)
{
	CInfoPBRec tempPB;

	if(file != 0L){
		tempPB.dirInfo.ioNamePtr=0L;
		tempPB.dirInfo.ioVRefNum=file->vRefNum;
		tempPB.dirInfo.ioFDirIndex=-1;
		tempPB.dirInfo.ioDrDirID=file->parID;
		if(PBGetCatInfoSync(&tempPB)==noErr){
			tempPB.dirInfo.ioDrMdDat=LMGetTime();
			tempPB.dirInfo.ioDrDirID=file->parID;
			PBSetCatInfoSync(&tempPB);
		}
	}
}
		
